/*
 * This tests a flow for the cloudron owner backs up a cloudron.
 * The backup is validated and restored from.
 */

'use strict';

var AppStore = require('../appstore.js'),
    assert = require('assert'),
    Cloudron = require('../cloudron.js'),
    common = require('../common.js'),
    dnsSync = require('dns-sync'),
    ImapProbe = require('../imap-probe.js'),
    mailer = require('../mailer.js'),
    sleep = require('../shell.js').sleep,
    _ = require('underscore');

require('colors');

process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
var BOX_VERSION = process.env.BOX_VERSION;

describe('Cloudron backup testing', function () {
    this.timeout(0);

    var appStore = new AppStore('https://api.staging.cloudron.io');

    var owner = common.getOwner();
    var admin = common.getAdmin();
    var cloudron, appId, box, backupInfo;

    var imap = new ImapProbe({
        user: process.env.IMAP_USERNAME,
        password: process.env.IMAP_PASSWORD,
        host: process.env.IMAP_HOST,
        port: 993, // imap port
        tls: true,
        readOnly: true
    });

    it('can login to the store', function () {
        var accessToken = appStore.getAccessToken(owner);
        appStore.setCredentials(owner.password, accessToken);

        var adminAccessToken = appStore.getAccessToken(admin);
        appStore.setAdminCredentials(admin.password, adminAccessToken);
    });

    it('can create a cloudron', function () {
        box = appStore.createCloudron({
            domain: common.cloudronDomain(__filename),
            region: 'sfo1',
            size: '1gb',
            version: BOX_VERSION
        });
        box = appStore.waitForCloudron(box.id);
        cloudron = new Cloudron(box);
    });

    it('can activate the box', function () {
        cloudron.activate(owner);
    });

    it('can login to the box', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    it('can enable email', function () {
        cloudron.setEmailEnabled(true);
    });

    // mail alias tests (do this here because alias takes sometime to propagate)
    it('can set aliases', function () {
        cloudron.setAliases(['mufti', 'mail']);
    });

    // mail tests as normal user
    it('can send mail to cloudron user', function (done) {
        mailer.sendMailToCloudronUser(owner.username + '@' + cloudron.fqdn(), done);
    });

    it('did receive mail', function (done) {
        cloudron.checkMail(owner, done);
    });

    it('can send mail from cloudron user', function (done) {
        cloudron.sendMail(owner, 'test+' + cloudron.fqdn() + '@cloudron.io', done);
    });

    it('did send mail from cloudron user', function (done) {
        imap.probe({
            subject: common.regexp('Hi from e2e test - ' + cloudron.fqdn()),
            to: common.regexp('test+' + cloudron.fqdn() + '@cloudron.io'),
            body: /This release depends on you/
        }, done);
    });

    it('can save a sieve script', function (done) {
        cloudron.saveSieveScript(owner, done);
    });

    // alias tests
    it('can send email to alias', function (done) {
        mailer.sendMailToCloudronUser('mufti@' + box.domain, done);
    });

    it('did receive mail as alias (using owner name creds)', function (done) {
        cloudron.checkMail(_.extend(owner, { to: 'mufti' }), done);
    });

    it('can send mail as alias', function (done) {
        cloudron.sendMail(_.extend(owner, { from: 'mufti' }), 'test+' + cloudron.fqdn() + '@cloudron.io', done);
    });

    it('did send mail as cloudron alias', function (done) {
        imap.probe({
            subject: common.regexp('Hi from e2e test - ' + cloudron.fqdn()),
            from: common.regexp('mufti@' + cloudron.fqdn()),
            to: common.regexp('test+' + cloudron.fqdn() + '@cloudron.io'),
            body: /This release depends on you/
        }, done);
    });

    // this is not implemented. cannot auth using alias
    xit('did receive mail to alias (using alias creds)', function (done) {
        cloudron.checkMail({ username: 'mufti', password: owner.password, to: 'mufti' }, done);
    });

    ////////////////////////// restore
    var location = 'test' + (Math.random() * 10000).toFixed();
    it('can install app', function () {
        var manifest = appStore.getManifest(common.TESTAPP_ID, common.TESTAPP_VERSION);
        appId = cloudron.installApp(location, manifest);
    });

    it('can populate the addons', function () {
        cloudron.populateAddons(cloudron.appFqdn(location));
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can backup the box', function () {
        backupInfo = cloudron.backup();
        assert.strictEqual(backupInfo.dependsOn.length, 1);
    });

    it('can restore the box', function () {
        appStore.restore(box.id, backupInfo.id + '.tar.gz');
        box = appStore.waitForCloudron(box.id);
    });

    it('wait for local dns', function () {
        var expectedIp = box.ip;

        for (var i = 0; i < 50; i++) {
            var ip = dnsSync.resolve(box.domain);
            if (ip === expectedIp) return;

             console.log('waiting for local dns to change from %s to %s', ip, expectedIp);
             sleep(30);
        }
    });

    it('can login to cloudron', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    // mail tests as normal user
    it('retains old mail', function (done) {
        cloudron.checkMail(owner, done);
    });

    it('can check sieve script', function (done) {
        cloudron.checkSieveScript(owner, done);
    });

    // alias tests continue to work after restore
    it('can send email to alias', function (done) {
        mailer.sendMailToCloudronUser('mufti@' + box.domain, done);
    });

    it('did receive mail as alias (using owner name creds)', function (done) {
        cloudron.checkMail(_.extend(owner, { to: 'mufti' }), done);
    });

    it('can send mail as alias', function (done) {
        cloudron.sendMail(_.extend(owner, { from: 'mufti' }), 'test+' + cloudron.fqdn() + '@cloudron.io', done);
    });

    it('did send mail as cloudron alias', function (done) {
        imap.probe({
            subject: common.regexp('Hi from e2e test - ' + cloudron.fqdn()),
            from: common.regexp('mufti@' + cloudron.fqdn()),
            to: common.regexp('test+' + cloudron.fqdn() + '@cloudron.io'),
            body: /This release depends on you/
        }, done);
    });

    it('wait for app', function () {
        cloudron.waitForApp(appId);
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can uninstall app', function () {
        cloudron.uninstallApp(appId);
    });

    it('can delete the cloudron', function () {
        appStore.deleteCloudron(box);
    });
});
